import json

class Destination():

    def __init__(self, session, form_id):
        self.r = session
        self.form_id = form_id
        self.base_url = 'https://www.devicemagic.com/api/forms/{0}/destinations'.format(self.form_id)

    def all(self):
        request = self.r.get(self.base_url + ".json")
        return request.json()

    def details(self, destination_id):
        request = self.r.get(self.base_url + "/" + str(destination_id) + ".json")
        return request.json()

    def create(self, json, form_id=None):
        if form_id != None:
            url = "https://www.devicemagic.com/api/forms/{0}/destinations".format(form_id)
        else:
            url = self.base_url
        headers = {'Content-Type': 'application/json'}    
        request = self.r.post(url, data=json, headers=headers)
        if request.status_code >= 200 and request.status_code < 300:
            return "Destination created"
        else:
            return "Failed with status code: {0}".format(request.status_code)

    def update(self, destination_id, json):
        headers = {'Content-Type': 'application/json'}
        request = self.r.put(self.base_url + "/" + str(destination_id) + ".json", data=json, headers=headers)
        if request.status_code >= 200 and request.status_code < 300:
            return "Destination updated"
        else:
            return "Failed with status code: {0}".format(request.status_code)

    def delete(self, destination_id):
        request = self.r.delete(self.base_url + "/" + str(destination_id))
        if request.status_code >= 200 and request.status_code < 300:
            return "Destination deleted"
        else:
            return "Failed with status code: {0}".format(request.status_code)

    def copy(self, destination_id, form_id=None):
        destination_to_copy = self.details(destination_id)
        format = destination_to_copy["destination"]["format_type"]
        transport = destination_to_copy["destination"]["transport_type"]
        binary = destination_to_copy["destination"]["binary_transport_type"]
        destination_elements = [format, transport, binary]
        self.__remove_keys(destination_elements, destination_to_copy)
        new_destination_json = self.__format_destination_json(destination_to_copy, format, transport, binary)
        return self.create(new_destination_json, form_id=form_id)

    def __remove_keys(self, destination_elements, json):
        for element in destination_elements:
            if element == None:
                continue
            del json["destination"][element]["id"]
            del json["destination"][element]["created_at"]
            del json["destination"][element]["updated_at"]

    def __format_destination_json(self, destination_to_copy, format, transport, binary):
        new_destination_json = {}
        new_destination_json["destination"] = {}
        new_destination_json["destination"]["description"] = destination_to_copy["destination"]["description"]
        new_destination_json["destination"]["format_selection"] = format
        new_destination_json["destination"]["transport_selection"] = transport
        new_destination_json["destination"]["binary_transport_selection"] = binary
        new_destination_json[format] = destination_to_copy["destination"][format]
        new_destination_json[transport] = destination_to_copy["destination"][transport]
        if binary != None:
            new_destination_json[binary] = destination_to_copy["destination"][binary]
        new_destination_json = json.dumps(new_destination_json)
        return new_destination_json


class Database():

    def __init__(self, session, database_id):
        self.r = session
        self.database_id = database_id
        self.base_url = 'https://www.devicemagic.com/api/forms/{0}/device_magic_database.json'.format(self.database_id)

    def json(self, *args):
        return self.__request_with_params(args) if args else self.__simple_request()
    
    def __simple_request(self):
        request = self.r.get(self.base_url) # For all submissions within the Device Magic Database
        return request.json()

    def __join_params(self, parameters):
        params = [ param.strip() for param in parameters ]
        params_for_url = "?" + "&".join(params)
        return params_for_url

    def __request_with_params(self, parameters):
        params = self.__join_params(parameters)
        request = self.r.get(self.base_url + params)
        return request.json()







from .database import Database
from .destination import Destination
from .device import Device
from .form import Form
from .resource import Resource
from .group import Group
from .dispatch import Dispatch
from .auth import DeviceMagicAuth
import requests

class DeviceMagic():

    def __init__(self, args={}):
        self.api_key = args.get('api_key')
        session = requests.Session()
        session.auth = DeviceMagicAuth(self.api_key)
        self.session = session
        self.form_id = str(args.get('form_id'))
        self.database_id = str(args.get('database_id'))
        self.org_id = str(args.get('org_id'))
        self.resource_id = str(args.get('resource_id'))
        self.file_path = args.get('file_path')
        self.device_identifier = args.get('device_identifier')
        self.database = Database(session, self.database_id)
        self.destination = Destination(session, self.form_id)
        self.device = Device(session, self.org_id)
        self.form = Form(session, self.org_id)
        self.resource = Resource(session, self.file_path)
        self.group = Group(session, self.org_id)
        self.dispatch = Dispatch(session, self.org_id)


class Dispatch():

    def __init__(self, session, org_id):
        self.r = session
        self.org_id = org_id
        self.device_base_url = 'https://www.devicemagic.com/clients'
        self.org_base_url = 'https://www.devicemagic.com/organizations'

    def all(self, device_identifier=None):
        if device_identifier != None:
            url = self.device_base_url + "/" + device_identifier + "/oneshots"
        else:
            url = self.org_base_url + "/" + self.org_id + "/oneshots"
        request = self.r.get(url)
        return request.text

    def push(self, device_identifier, json):
        headers = {'Content-Type': 'application/json'}  
        request = self.r.post(self.device_base_url + "/" + device_identifier + "/oneshots", data=json, headers=headers)
        if request.status_code >= 200 and request.status_code < 300:
            return "Dispatch successful"
        else:
            return "Failed with status code: {0}".format(request.status_code)
    
    def update(self, device_identifier, oneshot_id, json):
        headers = {'Content-Type': 'application/json'}
        request = self.r.put(self.device_base_url + "/" + device_identifier + "/oneshots/" + str(oneshot_id), data=json, headers=headers)
        if request.status_code >= 200 and request.status_code < 300:
            return "Dispatch updated"
        else:
            return "Failed with status code: {0}".format(request.status_code)

    def delete(self, device_identifier, oneshot_id=None):
        if oneshot_id != None:
            url = self.device_base_url + "/" + device_identifier + "/oneshots/" + str(oneshot_id)
            request = self.r.delete(url)
        else:
            url = self.device_base_url + "/" + device_identifier + "/oneshots/destroy_all"
            request = self.r.post(url)
        if request.status_code >= 200 and request.status_code < 300:
            return "Dispatch deleted"
        else:
            return "Failed with status code: {0}".format(request.status_code)


class Form():

    def __init__(self, session, org_id):
        self.r = session
        self.org_id = org_id
        self.base_url = 'https://www.devicemagic.com/organizations/{0}/forms'.format(self.org_id)

    def all(self):
        request = self.r.get(self.base_url + ".json")
        return request.json()

    def details(self, form_id):
        request = self.r.get(self.base_url + "/"+ str(form_id) + ".json")
        return request.json()

    def create(self, json):
        headers = {'Content-Type': 'application/json'}    
        request = self.r.post(self.base_url, data=json, headers=headers)
        if request.status_code >= 200 and request.status_code < 300:
            return "Form created"
        else:
            return "Failed with status code: {0}".format(request.status_code)

    def update(self, form_id, json):
        headers = {'Content-Type': 'application/json'}
        request = self.r.put(self.base_url + "/" + str(form_id), data=json, headers=headers)
        if request.status_code >= 200 and request.status_code < 300:
            return "Form updated"
        else:
            return "Failed with status code: {0}".format(request.status_code)

    def delete(self, form_id):
        request = self.r.delete(self.base_url + "/" + str(form_id))
        if request.status_code >= 200 and request.status_code < 300:
            return "Form deleted"
        else:
            return "Failed with status code: {0}".format(request.status_code)

    def new_group(self, form_id, json):
        headers = {'Content-Type': 'application/json'}
        request = self.r.post(self.base_url + "/" + str(form_id) + "/properties", data=json, headers=headers)
        if request.status_code >= 200 and request.status_code < 300:
            return "Form group updated"
        else:
            return "Failed with status code: {0}".format(request.status_code)
